import { Inject, Provide } from '@midwayjs/core';
import { LoginType, UPwdDTO } from '../../frame/Options';
import { get } from 'lodash';
import { ServiceError } from '../../error/CustomerError.error';
import { ErrorCode } from '../../error/Codes';
import { RoleService } from '../system/role.service';
import { InjectEntityModel } from '@midwayjs/typeorm';
import { Equal, Repository } from 'typeorm';
import * as bcrypt from 'bcryptjs';
import { Admin } from '../../entity/system/admin.entity';
import { User } from '../../entity/system/user.entity';

@Provide()
export class LoginService {
  @Inject()
  roleService: RoleService;
  @InjectEntityModel(Admin)
  adminModel: Repository<Admin>;
  @InjectEntityModel(User)
  userModel: Repository<User>;
  /**
   * 账密登录
   * @param data 用户名和密码
   * @param type 用户类型
   * @returns 用户信息/空值
   */
  async loginByAccount(data, type: LoginType) {
    let model;
    if (type === LoginType.Admin) model = this.adminModel;
    else model = this.userModel;
    const user = await model.createQueryBuilder('t').where('t.account = :account', { account: data.account }).addSelect('t.password').getOne();
    if (!user) throw new ServiceError(ErrorCode.USER_NOT_FOUND);
    await this.checkAccountCanLogin(user, type);
    const result = bcrypt.compareSync(data.password, user.password);
    if (!result) throw new ServiceError(ErrorCode.BAD_PASSWORD);
    return user;
  }

  /**
   * 检查用户是否可以登录(从用户本身和角色检查)
   * @param user 用户信息
   * @param type 用户类型
   */
  async checkAccountCanLogin(user, type: LoginType) {
    /**
     * 判断是否可以使用:
     * 管理员:
     *  超级管理员无视,直接往下;
     *  普通管理员需要查看
     *    角色是否能用
     *    is_use是不是'0'
     * 其他用户;
     *  角色是否能用
     *  is_use是不是'0'
     * 普通管理员和其他用户是一种判断
     */

    if (type === 'Admin') {
      if (get(user, 'is_super') === '1') {
        if (get(user, 'is_use') === '1') throw new ServiceError(ErrorCode.USER_IS_DISABLED);
      }
    } else {
      if (get(user, 'status') !== '1') throw new ServiceError(ErrorCode.USER_IS_DISABLED);
      const qb = { code: type, is_use: '0' };
      const role = await this.roleService.fetch(qb);
      if (!role) throw new ServiceError(ErrorCode.ROLE_IS_DISABLED);
    }
  }

  /**
   * 修改密码
   * @param data 修改密码对象
   * @param type 规定的登录类型
   * @returns 新密码
   */
  async updatePwd(data: UPwdDTO, type: LoginType) {
    let model;
    if (type === LoginType.Admin) model = this.adminModel;
    else model = this.userModel;
    const user = await model.findOne({ where: { id: Equal(get(data, 'id')) } });
    if (!user) new ServiceError(ErrorCode.USER_NOT_FOUND);
    // 处理密码
    let password = get(data, 'password');
    if (password) {
      const salt = bcrypt.genSaltSync(10);
      password = bcrypt.hashSync(password, salt);
    }
    await model.update({ id: get(data, 'id') }, { password });
  }
}
